@@ -4,7 +4,7 @@
  * This file is part of mpop, a POP3 client.
  *
  * Copyright (C) 2000, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011,
- * 2014, 2015, 2016
+ * 2014, 2015, 2016, 2018, 2019
  * Martin Lambers <marlam@marlam.de>
  * Martin Stenberg <martin@gnutiken.se> (passwordeval support)
  *
@@ -92,6 +92,7 @@ account_t *account_new(const char *conffile, const char *id)
     a->tls_priorities = NULL;
     a->proxy_host = NULL;
     a->proxy_port = 0;
+    a->source_ip = NULL;
     return a;
 }
 
@@ -173,6 +174,7 @@ account_t *account_copy(account_t *acc)
             acc->tls_priorities ? xstrdup(acc->tls_priorities) : NULL;
         a->proxy_host = acc->proxy_host ? xstrdup(acc->proxy_host) : NULL;
         a->proxy_port = acc->proxy_port;
+        a->source_ip = acc->source_ip ? xstrdup(acc->source_ip) : NULL;
     }
     return a;
 }
@@ -209,6 +211,7 @@ void account_free(void *a)
         free(p->tls_md5_fingerprint);
         free(p->tls_priorities);
         free(p->proxy_host);
+        free(p->source_ip);
         free(p);
     }
 }
@@ -390,7 +393,8 @@ int check_auth_arg(char *arg)
             || strcmp(arg, "gssapi") == 0
             || strcmp(arg, "external") == 0
             || strcmp(arg, "login") == 0
-            || strcmp(arg, "ntlm") == 0)
+            || strcmp(arg, "ntlm") == 0
+            || strcmp(arg, "oauthbearer") == 0)
     {
         l = strlen(arg);
         for (i = 0; i < l; i++)
@@ -581,6 +585,11 @@ void override_account(account_t *acc1, account_t *acc2)
     {
         acc1->proxy_port = acc2->proxy_port;
     }
+    if (acc2->mask & ACC_SOURCE_IP)
+    {
+        free(acc1->source_ip);
+        acc1->source_ip = acc2->source_ip ? xstrdup(acc2->source_ip) : NULL;
+    }
     acc1->mask |= acc2->mask;
 }
 
@@ -608,28 +617,6 @@ int check_account(account_t *acc, int retrmail, char **errstr)
         *errstr = xasprintf(_("tls_cert_file requires tls_key_file"));
         return CONF_ESYNTAX;
     }
-    if (acc->tls_nocertcheck && acc->tls_trust_file)
-    {
-        *errstr = xasprintf(
-                _("cannot use tls_trust_file with tls_certcheck turned off"));
-        return CONF_ESYNTAX;
-    }
-    if (acc->tls_nocertcheck
-            && (acc->tls_sha256_fingerprint
-                || acc->tls_sha1_fingerprint || acc->tls_md5_fingerprint))
-    {
-        *errstr = xasprintf(
-                _("cannot use tls_fingerprint with tls_certcheck turned off"));
-        return CONF_ESYNTAX;
-    }
-    if (acc->tls_trust_file
-            && (acc->tls_sha256_fingerprint
-                || acc->tls_sha1_fingerprint || acc->tls_md5_fingerprint))
-    {
-        *errstr = xasprintf(
-                _("cannot use both tls_trust_file and tls_fingerprint"));
-        return CONF_ESYNTAX;
-    }
     if (acc->tls && !acc->tls_trust_file
             && !acc->tls_sha256_fingerprint && !acc->tls_sha1_fingerprint
             && !acc->tls_md5_fingerprint && !acc->tls_nocertcheck)
@@ -664,6 +651,7 @@ int get_password_eval(const char *arg, char **buf, char **errstr)
 {
     FILE *eval;
     size_t l;
+    int have_more_data;
 
     *buf = NULL;
     *errstr = NULL;
@@ -675,33 +663,34 @@ int get_password_eval(const char *arg, char **buf, char **errstr)
         {
             errno = ENOMEM;
         }
-        *errstr = xasprintf("cannot evaluate '%s': %s", arg, strerror(errno));
+        *errstr = xasprintf(_("cannot evaluate '%s': %s"), arg, strerror(errno));
         return CONF_EIO;
     }
 
     *buf = xmalloc(LINEBUFSIZE);
     if (!fgets(*buf, LINEBUFSIZE, eval))
     {
-        *errstr = xasprintf("cannot read output of '%s'", arg);
+        *errstr = xasprintf(_("cannot read output of '%s'"), arg);
         pclose(eval);
         free(*buf);
         *buf = NULL;
         return CONF_EIO;
     }
+    have_more_data = (fgetc(eval) != EOF);
     pclose(eval);
 
     l = strlen(*buf);
     if (l > 0)
     {
-        if ((*buf)[l - 1] != '\n')
+        if ((*buf)[l - 1] != '\n' && have_more_data)
         {
-            *errstr = xasprintf("output of '%s' is longer than %d characters, "
-                    "or is not terminated by newline", arg, LINEBUFSIZE - 1);
+            *errstr = xasprintf(_("output of '%s' is longer than %d characters"),
+                    arg, LINEBUFSIZE - 1);
             free(*buf);
             *buf = NULL;
             return CONF_EIO;
         }
-        else
+        if ((*buf)[l - 1] == '\n')
         {
             (*buf)[l - 1] = '\0';
             if (l - 1 > 0 && (*buf)[l - 2] == '\r')
@@ -790,7 +779,7 @@ char *trim_string(const char *s)
  * whitespace) and an argument part (the word after the command).
  * Whitespace is ignored.
  * Sets the flag 'empty_line' if the line is empty.
- * Sets the flag 'eof' if EOF occured.
+ * Sets the flag 'eof' if EOF occurred.
  * On errors, 'empty_line' and 'eof', 'cmd' and 'arg' NULL.
  * On success, 'cmd' and 'arg' are allocated strings.
  * Used error codes: CONF_EIO, CONF_EPARSE
@@ -1009,6 +998,7 @@ int read_conffile(const char *conffile, FILE *f, list_t **acc_list,
                 list_free(copy_from);
                 *errstr = xasprintf(_("line %d: missing account name"), line);
                 e = CONF_ESYNTAX;
+                free(acc_id);
                 break;
             }
             if (strchr(acc_id, ':') || strchr(acc_id, ','))
@@ -1017,6 +1007,7 @@ int read_conffile(const char *conffile, FILE *f, list_t **acc_list,
                 *errstr = xasprintf(_("line %d: an account name must not "
                             "contain colons or commas"), line);
                 e = CONF_ESYNTAX;
+                free(acc_id);
                 break;
             }
             if (find_account(*acc_list, acc_id))
@@ -1024,8 +1015,9 @@ int read_conffile(const char *conffile, FILE *f, list_t **acc_list,
                 list_free(copy_from);
                 *errstr = xasprintf(
                         _("line %d: account %s was already defined"),
-                        line, arg);
+                        line, acc_id);
                 e = CONF_ESYNTAX;
+                free(acc_id);
                 break;
             }
             acc = account_copy(defaults);
@@ -1553,6 +1545,19 @@ int read_conffile(const char *conffile, FILE *f, list_t **acc_list,
                 }
             }
         }
+        else if (strcmp(cmd, "source_ip") == 0)
+        {
+            acc->mask |= ACC_SOURCE_IP;
+            free(acc->source_ip);
+            if (*arg == '\0')
+            {
+                acc->source_ip = NULL;
+            }
+            else
+            {
+                acc->source_ip = xstrdup(arg);
+            }
+        }
         else if (strcmp(cmd, "tls_force_sslv3") == 0)
         {
             /* compatibility with versions <= 1.0.29: silently ignore */
