@@ -4,7 +4,7 @@
  * This file is part of mpop, a POP3 client.
  *
  * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2014,
- * 2015, 2016
+ * 2015, 2016, 2018, 2019
  * Martin Lambers <marlam@marlam.de>
  * Dimitrios Apostolou <jimis@gmx.net> (UID handling)
  *
@@ -40,9 +40,8 @@
 
 #ifdef HAVE_LIBGSASL
 # include <gsasl.h>
-#else
-# include "base64.h"
 #endif
+#include "base64.h"
 #include "md5-apps.h"
 
 #include "gettext.h"
@@ -205,7 +204,7 @@ void pop3_session_free(pop3_session_t *session)
 
 int pop3_connect(pop3_session_t *session,
         const char *proxy_hostname, int proxy_port,
-        const char *server_hostname, int port, int timeout,
+        const char *server_hostname, int port, const char *source_ip, int timeout,
         const char **server_canonical_name, const char **server_address,
         char **errstr)
 {
@@ -213,7 +212,7 @@ int pop3_connect(pop3_session_t *session,
 
     session->server_hostname = xstrdup(server_hostname);
     e = net_open_socket(proxy_hostname, proxy_port,
-            server_hostname, port, timeout, &session->fd,
+            server_hostname, port, source_ip, timeout, &session->fd,
             &session->server_canonical_name, &session->server_address, errstr);
     if (server_canonical_name)
     {
@@ -235,7 +234,7 @@ int pop3_connect(pop3_session_t *session,
  * If a newline is read, it is stored into the buffer. A '\0' is stored after
  * the last character in the buffer. The length of the resulting string (the
  * number of characters excluding the terminating '\0') will be stored in 'len'.
- * If an error occured, '\0' is stored in buffer[0], and 'len' will be 0.
+ * If an error occurred, '\0' is stored in buffer[0], and 'len' will be 0.
  * Used error codes: POP3_EIO
  */
 
@@ -864,6 +863,10 @@ int pop3_capa(pop3_session_t *session, char **errstr)
                 {
                     session->cap.flags |= POP3_CAP_AUTH_NTLM;
                 }
+                if (strstr(session->buffer + 5, "OAUTHBEARER"))
+                {
+                    session->cap.flags |= POP3_CAP_AUTH_OAUTHBEARER;
+                }
             }
             else if (strncmp(session->buffer, "IMPLEMENTATION", 14) == 0)
             {
@@ -924,18 +927,20 @@ int pop3_capa(pop3_session_t *session, char **errstr)
 
 #ifdef HAVE_TLS
 int pop3_tls_init(pop3_session_t *session,
-        const char *tls_key_file, const char *tls_cert_file,
+        const char *tls_key_file, const char *tls_cert_file, const char *pin,
         const char *tls_trust_file, const char *tls_crl_file,
         const unsigned char *tls_sha256_fingerprint,
         const unsigned char *tls_sha1_fingerprint,
         const unsigned char *tls_md5_fingerprint,
         int min_dh_prime_bits, const char *priorities,
+        const char *hostname,
+        int no_certcheck,
         char **errstr)
 {
-    return tls_init(&session->tls, tls_key_file, tls_cert_file,
+    return tls_init(&session->tls, tls_key_file, tls_cert_file, pin,
             tls_trust_file, tls_crl_file,
             tls_sha256_fingerprint, tls_sha1_fingerprint, tls_md5_fingerprint,
-            min_dh_prime_bits, priorities, errstr);
+            min_dh_prime_bits, priorities, hostname, no_certcheck, errstr);
 }
 #endif /* HAVE_TLS */
 
@@ -983,11 +988,11 @@ int pop3_tls_stls(pop3_session_t *session, char **errmsg, char **errstr)
  */
 
 #ifdef HAVE_TLS
-int pop3_tls(pop3_session_t *session, const char *hostname, int tls_nocertcheck,
-        tls_cert_info_t *tci, char **errstr)
+int pop3_tls(pop3_session_t *session,
+        tls_cert_info_t *tci, char **tls_parameter_description, char **errstr)
 {
-    return tls_start(&session->tls, session->fd, hostname, tls_nocertcheck, tci,
-            errstr);
+    return tls_start(&session->tls, session->fd, tci,
+            tls_parameter_description, errstr);
 }
 #endif /* HAVE_TLS */
 
@@ -1653,16 +1658,10 @@ int pop3_retr_get_from_addr(pop3_session_t *session, FILE *f, char **from_addr,
 int pop3_write_received_header(pop3_session_t *session, FILE *f, int crlf,
         char **errstr)
 {
-    time_t t;
     char rfc2822_timestamp[32];
     int e;
 
-    if ((t = time(NULL)) < 0)
-    {
-        *errstr = xasprintf(_("cannot get system time: %s"), strerror(errno));
-        return POP3_ELIBFAILED;
-    }
-    print_time_rfc2822(t, rfc2822_timestamp);
+    print_time_rfc2822(time(NULL), rfc2822_timestamp);
 
     /* Write the Received header */
     e = (fprintf(f, "Received: from %s", session->server_hostname) < 0);
@@ -1804,7 +1803,7 @@ int pop3_delivery(pop3_session_t *session, volatile sig_atomic_t *abort,
             /* get the from address if needed */
             if (delivery->want_from_addr)
             {
-                if (!(tmpf = tempfile(PACKAGE_NAME)))
+                if (!(tmpf = tmpfile()))
                 {
                     *errstr = xasprintf(_("cannot create temporary file: %s"),
                             strerror(errno));
@@ -2243,6 +2242,77 @@ int pop3_auth_apop(pop3_session_t *session,
 }
 
 
+/*
+ * pop3_auth_oauthbearer()
+ *
+ * Do POP3 authentication via AUTH OAUTHBEARER.
+ * The POP3 server must support POP3_CAP_AUTH_OAUTHBEARER
+ * Used error codes: POP3_EIO, POP3_EAUTHFAIL, POP3_EINVAL
+ */
+
+int pop3_auth_oauthbearer(pop3_session_t *session,
+        const char *hostname, unsigned short port,
+        const char *user, const char *token,
+        char **errmsg, char **errstr)
+{
+    int e;
+    char *oauth;
+    size_t oa_len;
+    char *b64;
+    size_t b64_len;
+
+    oa_len = 4 + /* "n,a=" */
+             strlen(user) +
+             7 + /* ",^Ahost=" */
+             strlen(hostname) +
+             6 + 5 + /* "^Aport=" + up to 5 digits */
+             13 + /* "^Aauth=Bearer " */
+             strlen(token) +
+             2 /* "^A^A" */;
+    oauth = xmalloc(oa_len + 1);
+    oa_len = snprintf(oauth, oa_len + 1,
+                      "n,a=%s,\001host=%s\001port=%d\001auth=Bearer %s\001\001",
+                      user, hostname, port, token);
+    b64_len = BASE64_LENGTH(oa_len) + 1;
+    b64 = xmalloc(b64_len);
+    base64_encode(oauth, oa_len, b64, b64_len);
+    e = pop3_send_cmd(session, errstr, "AUTH OAUTHBEARER %s", b64);
+    free(oauth);
+    free(b64);
+    if (e != POP3_EOK)
+    {
+        return e;
+    }
+    if ((e = pop3_get_msg(session, 1, errstr)) != POP3_EOK)
+    {
+        return e;
+    }
+    if (session->buffer[0] == '+' && session->buffer[1] == 'O' && session->buffer[2] == 'K')
+    {
+        return POP3_EOK;
+    }
+    if (session->buffer[0] == '+' && session->buffer[1] == ' ')
+    {
+        if ((e = pop3_send_cmd(session, errstr, "")) != POP3_EOK)
+        {
+            return e;
+        }
+        if ((e = pop3_get_msg(session, 1, errstr)) != POP3_EOK)
+        {
+            return e;
+        }
+    }
+    if (!pop3_msg_ok(session->buffer))
+    {
+        *errmsg = xstrdup(session->buffer);
+        *errstr = xasprintf(_("authentication failed (method %s)"), "OAUTHBEARER");
+        return POP3_EAUTHFAIL;
+    }
+
+    return POP3_EOK;
+}
+
+
 /*
  * pop3_auth_plain()
  *
@@ -2498,7 +2568,7 @@ int pop3_auth_external(pop3_session_t *session, const char *user,
     {
         return e;
     }
-    if ((e = pop3_get_msg(session, 0, errstr)) != POP3_EOK)
+    if ((e = pop3_get_msg(session, 1, errstr)) != POP3_EOK)
     {
         return e;
     }
@@ -2561,7 +2631,9 @@ int pop3_server_supports_authmech(pop3_session_t *session, const char *mech)
             || ((session->cap.flags & POP3_CAP_AUTH_LOGIN)
                 && strcmp(mech, "LOGIN") == 0)
             || ((session->cap.flags & POP3_CAP_AUTH_NTLM)
-                && strcmp(mech, "NTLM") == 0));
+                && strcmp(mech, "NTLM") == 0)
+            || ((session->cap.flags & POP3_CAP_AUTH_OAUTHBEARER)
+                && strcmp(mech, "OAUTHBEARER") == 0));
 }
 
 
@@ -2578,7 +2650,8 @@ int pop3_client_supports_authmech(const char *mech)
     int supported = 0;
     Gsasl *ctx;
 
-    if (strcmp(mech, "USER") == 0 || strcmp(mech, "APOP") == 0)
+    if (strcmp(mech, "USER") == 0 || strcmp(mech, "APOP") == 0
+            || strcmp(mech, "OAUTHBEARER") == 0)
     {
         supported = 1;
     }
@@ -2600,7 +2673,8 @@ int pop3_client_supports_authmech(const char *mech)
             || strcmp(mech, "CRAM-MD5") == 0
             || strcmp(mech, "PLAIN") == 0
             || strcmp(mech, "EXTERNAL") == 0
-            || strcmp(mech, "LOGIN") == 0);
+            || strcmp(mech, "LOGIN") == 0
+            || strcmp(mech, "OAUTHBEARER") == 0);
 
 #endif /* not HAVE_LIBGSASL */
 }
@@ -2617,6 +2691,7 @@ int pop3_auth(pop3_session_t *session,
         const char *user,
         const char *password,
         const char *hostname,
+        unsigned short port,
         const char *ntlmdomain,
         char *(*password_callback)(const char *hostname, const char *user),
         char **errmsg,
@@ -2657,37 +2732,37 @@ int pop3_auth(pop3_session_t *session,
     {
         /* Choose "best" authentication mechanism. */
         /* TODO: use gsasl_client_suggest_mechanism()? */
-        if (gsasl_client_support_p(ctx, "SCRAM-SHA-1")
-                && (session->cap.flags & POP3_CAP_AUTH_SCRAM_SHA_1))
-        {
-            auth_mech = "SCRAM-SHA-1";
-        }
-        else if (gsasl_client_support_p(ctx, "CRAM-MD5")
-                && (session->cap.flags & POP3_CAP_AUTH_CRAM_MD5))
-        {
-            auth_mech = "CRAM-MD5";
-        }
 #ifdef HAVE_TLS
-        else if (tls_is_active(&session->tls))
+        if (tls_is_active(&session->tls))
         {
-            if (session->cap.flags & POP3_CAP_AUTH_APOP)
-            {
-                auth_mech = "APOP";
-            }
-            else if (gsasl_client_support_p(ctx, "PLAIN")
+            if (gsasl_client_support_p(ctx, "PLAIN")
                     && (session->cap.flags & POP3_CAP_AUTH_PLAIN))
             {
                 auth_mech = "PLAIN";
             }
+            else if (session->cap.flags & POP3_CAP_AUTH_APOP)
+            {
+                auth_mech = "APOP";
+            }
+            else if (session->cap.flags & POP3_CAP_AUTH_USER)
+            {
+                auth_mech = "USER";
+            }
+            else if (gsasl_client_support_p(ctx, "SCRAM-SHA-1")
+                    && (session->cap.flags & POP3_CAP_AUTH_SCRAM_SHA_1))
+            {
+                auth_mech = "SCRAM-SHA-1";
+            }
+            else if (gsasl_client_support_p(ctx, "CRAM-MD5")
+                    && (session->cap.flags & POP3_CAP_AUTH_CRAM_MD5))
+            {
+                auth_mech = "CRAM-MD5";
+            }
             else if (gsasl_client_support_p(ctx, "DIGEST-MD5")
                     && (session->cap.flags & POP3_CAP_AUTH_DIGEST_MD5))
             {
                 auth_mech = "DIGEST-MD5";
             }
-            else if (session->cap.flags & POP3_CAP_AUTH_USER)
-            {
-                auth_mech = "USER";
-            }
             else if (gsasl_client_support_p(ctx, "LOGIN")
                     && (session->cap.flags & POP3_CAP_AUTH_LOGIN))
             {
@@ -2699,7 +2774,15 @@ int pop3_auth(pop3_session_t *session,
                 auth_mech = "NTLM";
             }
         }
+        else
 #endif /* HAVE_TLS */
+        {
+            if (gsasl_client_support_p(ctx, "SCRAM-SHA-1")
+                    && (session->cap.flags & POP3_CAP_AUTH_SCRAM_SHA_1))
+            {
+                auth_mech = "SCRAM-SHA-1";
+            }
+        }
     }
     if (strcmp(auth_mech, "") == 0)
     {
@@ -2723,8 +2806,7 @@ int pop3_auth(pop3_session_t *session,
     /* Check availability of required authentication data */
     if (strcmp(auth_mech, "EXTERNAL") != 0)
     {
-        /* GSSAPI, SCRAM-SHA-1, DIGEST-MD5, CRAM-MD5, PLAIN, LOGIN, NTLM, USER,
-         * APOP all need a user name */
+        /* All authentication schemes need a user name */
         if (!user)
         {
             gsasl_done(ctx);
@@ -2732,8 +2814,8 @@ int pop3_auth(pop3_session_t *session,
                     auth_mech);
             return POP3_EUNAVAIL;
         }
-        /* SCRAM-SHA-1, DIGEST-MD5, CRAM-MD5, PLAIN, LOGIN, NTLM, USER, APOP
-         * all need a password */
+        /* SCRAM-SHA-1, DIGEST-MD5, CRAM-MD5, PLAIN, LOGIN, NTLM, USER, APOP,
+         * OAUTHBEARER all need a password */
         if (strcmp(auth_mech, "GSSAPI") != 0 && !password)
         {
             if (!password_callback
@@ -2749,7 +2831,7 @@ int pop3_auth(pop3_session_t *session,
         }
     }
 
-    /* USER and APOP are built-in, all other methods are provided by GNU SASL */
+    /* USER, APOP, OAUTHBEARER are built-in, all other methods are provided by GNU SASL */
     if (strcmp(auth_mech, "USER") == 0)
     {
         gsasl_done(ctx);
@@ -2764,6 +2846,13 @@ int pop3_auth(pop3_session_t *session,
         free(callback_password);
         return e;
     }
+    else if (strcmp(auth_mech, "OAUTHBEARER") == 0)
+    {
+        gsasl_done(ctx);
+        e = pop3_auth_oauthbearer(session, hostname, port, user, password, errmsg, errstr);
+        free(callback_password);
+        return e;
+    }
     else if ((error_code = gsasl_client_start(ctx, auth_mech, &sctx))
             != GSASL_OK)
     {
@@ -2928,25 +3017,25 @@ int pop3_auth(pop3_session_t *session,
     /* Choose "best" authentication mechanism. */
     if (strcmp(auth_mech, "") == 0)
     {
-        if (session->cap.flags & POP3_CAP_AUTH_CRAM_MD5)
-        {
-            auth_mech = "CRAM-MD5";
-        }
-        else if (session->cap.flags & POP3_CAP_AUTH_APOP)
-        {
-            auth_mech = "APOP";
-        }
 #ifdef HAVE_TLS
-        else if (tls_is_active(&session->tls))
+        if (tls_is_active(&session->tls))
         {
             if (session->cap.flags & POP3_CAP_AUTH_PLAIN)
             {
                 auth_mech = "PLAIN";
             }
+            else if (session->cap.flags & POP3_CAP_AUTH_APOP)
+            {
+                auth_mech = "APOP";
+            }
             else if (session->cap.flags & POP3_CAP_AUTH_USER)
             {
                 auth_mech = "USER";
             }
+            else if (session->cap.flags & POP3_CAP_AUTH_CRAM_MD5)
+            {
+                auth_mech = "CRAM-MD5";
+            }
             else if (session->cap.flags & POP3_CAP_AUTH_LOGIN)
             {
                 auth_mech = "LOGIN";
@@ -2975,7 +3064,7 @@ int pop3_auth(pop3_session_t *session,
     /* Check availability of required authentication data */
     if (strcmp(auth_mech, "EXTERNAL") != 0)
     {
-        /* CRAM-MD5, PLAIN, LOGIN, APOP, USER all need a user name and
+        /* CRAM-MD5, PLAIN, LOGIN, APOP, USER, OAUTHBEARER all need a user name and
          * password */
         if (!user)
         {
@@ -3021,6 +3110,10 @@ int pop3_auth(pop3_session_t *session,
     {
         e = pop3_auth_login(session, user, password, errmsg, errstr);
     }
+    else if (strcmp(auth_mech, "OAUTHBEARER") == 0)
+    {
+        e = pop3_auth_oauthbearer(session, hostname, port, user, password, errmsg, errstr);
+    }
     else
     {
         *errstr = xasprintf(_("authentication method %s not supported"),
@@ -3106,3 +3199,32 @@ void pop3_close(pop3_session_t *session)
 #endif /* HAVE_TLS */
     net_close_socket(session->fd);
 }
+
+
+/*
+ * pop3_exitcode()
+ *
+ * see pop3.h
+ */
+
+int pop3_exitcode(int pop3_error_code)
+{
+    switch (pop3_error_code)
+    {
+        case POP3_EIO:
+        case POP3_EDELIVERY:
+            return EX_IOERR;
+        case POP3_EPROTO:
+            return EX_PROTOCOL;
+        case POP3_EINVAL:
+            return EX_DATAERR;
+        case POP3_EAUTHFAIL:
+            return EX_NOPERM;
+        case POP3_EINSECURE:
+        case POP3_EUNAVAIL:
+            return EX_UNAVAILABLE;
+        case POP3_ELIBFAILED:
+        default:
+            return EX_SOFTWARE;
+    }
+}
