@@ -4,7 +4,7 @@
  * This file is part of mpop, a POP3 client.
  *
  * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2014, 2015,
- * 2016
+ * 2016, 2018, 2019
  * Martin Lambers <marlam@marlam.de>
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -69,16 +69,17 @@
 #define POP3_CAP_AUTH_EXTERNAL          (1 << 7)
 #define POP3_CAP_AUTH_LOGIN             (1 << 8)
 #define POP3_CAP_AUTH_NTLM              (1 << 9)
-#define POP3_CAP_CAPA                   (1 << 10)
-#define POP3_CAP_TOP                    (1 << 11)
-#define POP3_CAP_LOGIN_DELAY            (1 << 12)
-#define POP3_CAP_PIPELINING             (1 << 13)
-#define POP3_CAP_EXPIRE                 (1 << 14)
-#define POP3_CAP_UIDL                   (1 << 15)
-#define POP3_CAP_IMPLEMENTATION         (1 << 16)
-#define POP3_CAP_STLS                   (1 << 17)
-#define POP3_CAP_RESP_CODES             (1 << 18)
-#define POP3_CAP_AUTH_RESP_CODE         (1 << 19)
+#define POP3_CAP_AUTH_OAUTHBEARER       (1 << 10)
+#define POP3_CAP_CAPA                   (1 << 11)
+#define POP3_CAP_TOP                    (1 << 12)
+#define POP3_CAP_LOGIN_DELAY            (1 << 13)
+#define POP3_CAP_PIPELINING             (1 << 14)
+#define POP3_CAP_EXPIRE                 (1 << 15)
+#define POP3_CAP_UIDL                   (1 << 16)
+#define POP3_CAP_IMPLEMENTATION         (1 << 17)
+#define POP3_CAP_STLS                   (1 << 18)
+#define POP3_CAP_RESP_CODES             (1 << 19)
+#define POP3_CAP_AUTH_RESP_CODE         (1 << 20)
 
 /*
  * Buffer size for communication with the POP3 server. This restricts the length
@@ -239,7 +240,7 @@ void pop3_session_free(pop3_session_t *session);
  */
 int pop3_connect(pop3_session_t *session,
         const char *proxy_hostname, int proxy_port,
-        const char *server_hostname, int port, int timeout,
+        const char *server_hostname, int port, const char *source_ip, int timeout,
         const char **server_canonical_name, const char **server_address,
         char **errstr);
 
@@ -281,12 +282,14 @@ int pop3_capa(pop3_session_t *session, char **errstr);
  */
 #ifdef HAVE_TLS
 int pop3_tls_init(pop3_session_t *session,
-        const char *tls_key_file, const char *tls_cert_file,
+        const char *tls_key_file, const char *tls_cert_file, const char *pin,
         const char *tls_trust_file, const char *tls_crl_file,
         const unsigned char *tls_sha256_fingerprint,
         const unsigned char *tls_sha1_fingerprint,
         const unsigned char *tls_md5_fingerprint,
         int min_dh_prime_bits, const char *priorities,
+        const char *hostname,
+        int tls_nocertcheck,
         char **errstr);
 #endif /* HAVE_TLS */
 
@@ -315,8 +318,8 @@ int pop3_tls_stls(pop3_session_t *session, char **errmsg, char **errstr);
  * Used error codes: TLS_ELIBFAILED, TLS_ECERT, TLS_EHANDSHAKE
  */
 #ifdef HAVE_TLS
-int pop3_tls(pop3_session_t *session, const char *hostname, int tls_nocertcheck,
-        tls_cert_info_t *tci, char **errstr);
+int pop3_tls(pop3_session_t *session,
+        tls_cert_info_t *tci, char **tls_parameter_description, char **errstr);
 #endif /* HAVE_TLS */
 
 /*
@@ -347,6 +350,8 @@ int pop3_server_supports_authmech(pop3_session_t *session, const char *mech);
  * POP3_EINSECURE.
  * The hostname is the name of the POP3 server. It may be needed for
  * authentication.
+ * The port is port number POP3 server accepts connections on. It may be needed
+ * for authentication.
  * The ntlmdomain may be NULL (even if you use NTLM authentication).
  * If 'password' is NULL, but the authentication method needs a password,
  * the 'password_callback' function is called (if 'password_callback' is not
@@ -361,6 +366,7 @@ int pop3_auth(pop3_session_t *session,
         const char *user,
         const char *password,
         const char *hostname,
+        unsigned short port,
         const char *ntlmdomain,
         char *(*password_callback)(const char *hostname, const char *user),
         char **errmsg,
@@ -420,7 +426,7 @@ int pop3_list(pop3_session_t *session, volatile sig_atomic_t *abort,
  * 0   - proceed normally
  * 1   - change message action to POP3_MSG_ACTION_DELETE
  * 2   - change message action to POP3_MSG_ACTION_IGNORE
- * >=3 - an error occured. Exit codes from sysexits.h are supported.
+ * >=3 - an error occurred. Exit codes from sysexits.h are supported.
  * If the action changed and filter_output is not NULL, the output function will
  * be called. 'data' can point to arbitrary user data that is passed to the
  * output function.
@@ -451,7 +457,7 @@ int pop3_filter(pop3_session_t *session, volatile sig_atomic_t *abort,
  * - 'progress' 99 times, while retrieving the mail, with 'percent' increasing
  *    from 1 to 99
  * - 'progress_end' once, after everything was retrieved successfully
- * - 'progress_abort' only if an error occured, to clean up the output
+ * - 'progress_abort' only if an error occurred, to clean up the output
  * If 'abort' is externally set, this function will abort and return
  * POP3_EABORTED. The POP3 session is not usable thereafter.
  * Used error codes: POP3_EIO, POP3_EPROTO, POP3_EINVAL, POP3_EDELIVERY,
@@ -509,4 +515,11 @@ int pop3_quit(pop3_session_t *session, char **errmsg, char **errstr);
  */
 void pop3_close(pop3_session_t *session);
 
+/*
+ * pop3_exitcode()
+ *
+ * Translate POP3_* error code to an error code from sysexits.h
+ */
+int pop3_exitcode(int pop3_error_code);
+
 #endif
